#Code 6.3  User-defined function for automated phase diagram

#Author:  Miles Medina (2016), Unpublished PhD dissertation, Appendix 2, 
#  University of Florida, Department of Agricultural and Biological Engineering.

#Inputs
#M:           Matrix whose columns are at least three time series.
#start.angle: Initial viewing angle for plot (degrees).
#step.angle:  Rotation rate (degrees of rotation per frame). Fractional values between
#             0 and 1 provide slow rotation. Set to 0 for no rotation.

animat3d.plot <- function(M,start.angle,step.angle){
    library(animation); library(scatterplot3d)
    ff.loc<-"../User Defined Functions/ffmpeg"
    #Determine axes' limits for plot
    x.min<-min(M[,1]); x.max<-max(M[,1])
    y.min<-min(M[,2]); y.max<-max(M[,2])
    z.min<-min(M[,3]); z.max<-max(M[,3])

    saveVideo(
    expr={
      ani.record(reset=TRUE) # discard previous plots
      ani.options(interval=0.05) # set interval between frames (seconds)
      for(i in 1:(dim(M)[1]-1)){
        # generate a new plot including each next row from matrix
        scatterplot3d(M[1:(i+1),1],M[1:(i+1),2],M[1:(i+1),3],box=F,
                      angle=(start.angle+step.angle*i),
                      xlim=c(x.min,x.max), ylim=c(y.min,y.max), zlim=c(z.min,z.max),
                      xlab="x",ylab="y",zlab="z",
                      type='l',lwd=2,color='blue')
        ani.record() # record the current plot as an animation frame
      }
    },
    video.name="animation.mp4",
    ffmpeg=ff.loc
    )
}

#Command to run user-defined function
setwd("../results")  #export video to "Results" directory
#animat3d.plot(Mx,20,0.5)